#include <cmath>
#include <msp/gl/matrix.h>
#include <msp/gl/misc.h>
#include <msp/gl/renderer.h>
#include "endpoint.h"
#include "layout.h"
#include "track.h"

using namespace Msp;

namespace R2C2 {

Endpoint3D::Endpoint3D(const Track3D &t, unsigned i):
	track(t),
	index(i),
	mesh(track.get_layout().get_catalogue().get_endpoint_mesh(track.get_track().get_type().get_appearance()))
{
	track.get_layout().get_endpoint_scene().add(*this);
}

Endpoint3D::~Endpoint3D()
{
	track.get_layout().get_endpoint_scene().remove(*this);
}

void Endpoint3D::render(GL::Renderer &renderer, const GL::Tag &tag) const
{
	if(tag=="unlit")
	{
		Snap sn = track.get_track().get_snap_node(index);

		GL::MatrixStack::Push push_mtx(renderer.matrix_stack());
		GL::Matrix matrix;
		matrix.translate(sn.position);
		matrix.rotate(sn.rotation+Angle::half_turn(), 0, 0, 1);
		renderer.matrix_stack() *= matrix;

		if(track.get_track().get_link(index))
			glColor4f(0.5, 0, 1, 0.5);
		else
			glColor4f(1, 0, 0.5, 0.5);

		mesh.draw(renderer);
	}
}

} // namespace R2C2
