#ifndef R2C2_3D_OVERLAY_H_
#define R2C2_3D_OVERLAY_H_

#include <msp/gl/camera.h>
#include <msp/gl/font.h>
#include <msp/gl/mesh.h>
#include <msp/gl/renderable.h>

namespace R2C2 {

class Object3D;

class Overlay3D: public Msp::GL::Renderable
{
private:	
	struct Icon
	{
		std::vector<const Msp::GL::Mesh *> graphics;
		std::string label;
		float width;
		Msp::GL::Mesh background;
		Msp::GL::Mesh text;

		Icon();
	};

	const Msp::GL::Font &font;
	std::map<std::string, Msp::GL::Mesh *> graphics;
	std::map<const Object3D *, Icon *> icons;

public:
	Overlay3D(const Msp::GL::Font &);
	~Overlay3D();

	void set_label(const Object3D &, const std::string &);
	void add_graphic(const Object3D &, const std::string &);
	void remove_graphic(const Object3D &, const std::string &);
	void clear_graphics(const Object3D &);
	void clear(const Object3D &);

	virtual void render(Msp::GL::Renderer &, const Msp::GL::Tag &) const;

private:
	Icon &get_icon(const Object3D &);
	const Msp::GL::Mesh *get_graphic(const std::string &);
	void update_icon(Icon &);
};

} // namespace R2C2

#endif
