#include <sigc++/bind.h>
#include <msp/input/keys.h>
#include "designer.h"
#include "tool.h"

using namespace Msp;

Tool::Tool(Designer &d, Input::Keyboard &keyboard, Input::Mouse &mouse):
	designer(d),
	shift_held(false),
	ctrl_held(false),
	done(false),
	accepted(false)
{
	keyboard.signal_button_press.connect(sigc::bind_return(sigc::mem_fun(this, &Tool::key_press), false));
	keyboard.signal_button_release.connect(sigc::bind_return(sigc::mem_fun(this, &Tool::key_release), false));
	mouse.signal_button_press.connect(sigc::bind_return(sigc::mem_fun(this, &Tool::button_press), false));
	mouse.signal_button_release.connect(sigc::bind_return(sigc::mem_fun(this, &Tool::button_release), false));
	mouse.signal_axis_motion.connect(sigc::bind_return(sigc::mem_fun(this, &Tool::axis_motion), false));

	pointer.x = mouse.get_axis_value(0);
	pointer.y = mouse.get_axis_value(1);
	ground_pointer = designer.map_pointer_to_ground(pointer);
}

void Tool::set_status(const std::string &s)
{
	status = s;
	signal_status.emit(status);
}

void Tool::set_done(bool a)
{
	accepted = a;
	done = true;
	signal_done.emit();
}

void Tool::key_press(unsigned key)
{
	if(key==Input::KEY_ESC)
		set_done(false);
	else if(key==Input::KEY_SHIFT_L || key==Input::KEY_SHIFT_R)
		shift_held = true;
	else if(key==Input::KEY_CTRL_L || key==Input::KEY_CTRL_R)
		ctrl_held = true;
}

void Tool::key_release(unsigned key)
{
	if(key==Input::KEY_SHIFT_L || key==Input::KEY_SHIFT_R)
		shift_held = false;
	else if(key==Input::KEY_CTRL_L || key==Input::KEY_CTRL_R)
		ctrl_held = false;
}

void Tool::button_press(unsigned btn)
{
	if(btn==1)
	{
		finish();
		set_done(true);
	}
	else if(btn==3)
		set_done(false);
}

void Tool::axis_motion(unsigned axis, float value, float)
{
	if(axis==0)
		pointer.x = value;
	else if(axis==1)
		pointer.y = value;
	ground_pointer = designer.map_pointer_to_ground(pointer);

	pointer_motion();
}
