#ifndef ENGINEER_H_
#define ENGINEER_H_

#include <msp/core/application.h>
#include <msp/graphics/simplewindow.h>
#include <msp/gl/camera.h>
#include <msp/gl/light.h>
#include <msp/gl/lighting.h>
#include <msp/gl/pipeline.h>
#include <msp/input/keyboard.h>
#include <msp/input/mouse.h>
#include "libr2c2/catalogue.h"
#include "libr2c2/train.h"
#include "3d/layout.h"
#include "3d/overlay.h"
#include "3d/path.h"
#include "3d/trackchain.h"
#include "3d/view.h"
#include "network/server.h"
#include "options.h"
#include "userinterface.h"

class Engineer: public Msp::RegisteredApplication<Engineer>
{
private:
	Options options;

	Msp::Graphics::SimpleGLWindow window;
	Msp::Input::Keyboard keyboard;
	Msp::Input::Mouse mouse;

	R2C2::Catalogue catalogue;
	R2C2::Layout layout;
	R2C2::Layout3D layout_3d;
	R2C2::Server *server;
	Msp::IO::EventDispatcher event_disp;
	R2C2::Overlay3D *overlay;
	std::map<R2C2::Train *, Msp::GL::Color> train_colors;

	R2C2::View3D main_view;
	UserInterface ui;

	std::list<R2C2::Train *> new_trains;
	R2C2::Vector pointer;
	std::list<R2C2::TrackChain3D *> emergencies;
	Msp::Time::TimeStamp emergency_blink_timeout;
	unsigned emergency_blink_state;

public:
	Engineer(int argc, char **argv);
	~Engineer();

	const R2C2::Catalogue &get_catalogue() const { return catalogue; }
	R2C2::Layout &get_layout() { return layout; }
	R2C2::Layout3D &get_layout_3d() { return layout_3d; }
	R2C2::View3D &get_main_view() { return main_view; }
	UserInterface &get_user_interface() { return ui; }
	int main();
	void quit() { exit(0); }
private:
	void tick();
	void button_press(unsigned);
	void axis_motion(unsigned, float, float);
	void view_all();
	R2C2::Object *pick_object(const R2C2::Vector &);
	void emergency(R2C2::Block *, const std::string &);
	void halt_event(bool);
	void train_added(R2C2::Train &);
	void process_new_train(R2C2::Train &);
	virtual void sighandler(int);
};

#endif
