#ifndef R2C2_NET_SERVER_H_
#define R2C2_NET_SERVER_H_

#include <msp/io/eventdispatcher.h>
#include <msp/net/communicator.h>
#include <msp/net/streamsocket.h>
#include <msp/net/streamserversocket.h>
#include "libr2c2/layout.h"
#include "libr2c2/trainai.h"
#include "packets.h"
#include "protocol.h"

namespace R2C2 {

class Server
{
private:
	struct Connection: Msp::Net::PacketReceiver<DriverStatePacket>,
		Msp::Net::PacketReceiver<TrainControlPacket>,
		Msp::Net::PacketReceiver<TrainFunctionPacket>,
		Msp::Net::PacketReceiver<TrainRoutePacket>
	{
		Server &server;
		Msp::Net::StreamSocket *socket;
		Msp::Net::Communicator comm;
		bool stale;

		Connection(Server &, Msp::Net::StreamSocket *);
		~Connection();

		void handshake_done();
		void end_of_file();
		virtual void receive(const DriverStatePacket &);
		virtual void receive(const TrainControlPacket &);
		virtual void receive(const TrainFunctionPacket &);
		virtual void receive(const TrainRoutePacket &);
		void comm_error(const std::exception &);
		void error(const std::string &);
	};

	Protocol proto;
	Layout &layout;
	Msp::Net::StreamServerSocket listen_sock;
	Msp::IO::EventDispatcher *event_disp;
	std::vector<Connection *> connections;

public:
	Server(Layout &);
	~Server();

	void use_event_dispatcher(Msp::IO::EventDispatcher &);
private:
	void incoming_connection();

	void driver_state_changed();
	void emergency(Block *, const std::string &);
	void train_added(Train &);
	void train_control_changed(const Train &, const std::string &, float);
	void train_function_changed(const Train &, unsigned, bool);
	void train_ai_event(const Train &, TrainAI &, const TrainAI::Message &);

	template<typename P>
	void send(const P &);
};

} // namespace R2C2

#endif
